@@ -26,6 +26,7 @@
 #'     tidy(lmm1)
 #'     tidy(lmm1, effects = "fixed")
 #'     tidy(lmm1, conf.int = TRUE)
+#'     tidy(lmm1, conf.int = TRUE, conf.level = 0.8)
 #'     tidy(lmm1, effects = "ran_pars")
 #'     tidy(lmm1, effects = "ran_vals")
 #'     tidy(lmm1, effects = "ran_coefs")
@@ -117,7 +118,7 @@ tidy.lme <- function(x, effects = c("var_model", "ran_pars", "fixed"),
       rename_regex_match() %>%
       tibblify("term")
     if (conf.int) {
-      cifix <- intervals(x, which = "fixed")[["fixed"]] %>%
+      cifix <- intervals(x, level = conf.level, which = "fixed")[["fixed"]] %>%
         data.frame() %>%
         dplyr::select(lower, upper) %>%
         setNames(c("conf.low", "conf.high")) %>%
@@ -141,99 +142,17 @@ tidy.lme <- function(x, effects = c("var_model", "ran_pars", "fixed"),
     if (!rscale %in% c("sdcor", "vcov")) {
       stop(sprintf("unrecognized ran_pars scale %s", sQuote(rscale)))
     }
-    grplen <- attr(x$modelStruct$reStruct, "plen")
-    multilevel <- (length(grplen) > 1)
     nonlin <- inherits(x, "nlme")
-
-    if (multilevel || nonlin) {
-      warning(
-        "ran_pars not yet implemented for ",
-        if (multilevel) {
-          "multiple levels of nesting"
-        } else {
-          "nonlinear models"
-        }
-      )
-      ret <- dplyr::tibble()
+    grplen <- attr(x$modelStruct$reStruct, "plen")
+    multilevel <- length(grplen) > 1
+    ## FIXME: work on multilevel models
+    if (nonlin || multilevel) {
+        warning("ran_pars not yet implemented for nonlinear or multilevel models")
+        ret <- dplyr::tibble()
     } else {
-      vc <- nlme::getVarCov(x)
-      ran_prefix <- switch(rscale,
-        vcov = c("var", "cov"),
-        sdcor = c("sd", "cor")
-      )
-      ## construct appropriate sd/cor names from dimnames
-      nmvec <- outer(
-        colnames(vc), rownames(vc),
-        function(x, y) {
-          ifelse(x == y,
-            sprintf(
-              "%s_%s",
-              ran_prefix[1],
-              x
-            ),
-            sprintf(
-              "%s_%s.%s",
-              ran_prefix[2],
-              x, y
-            )
-          )
-        }
-      )
-      lwrtri <- function(x) {
-        x[lower.tri(x, diag = TRUE)]
-      }
-      nmvec <- c(
-        lwrtri(nmvec),
-        sprintf("%s_%s", ran_prefix[1], "Observation")
-      )
-      grpnames <- c(rep(names(grplen), grplen), "Residual")
-      if (rscale == "vcov") {
-        vals <- c(lwrtri(vc), sigma(x)^2)
-      } else {
-        vals <- cov2cor(vc)
-        diag(vals) <- sqrt(diag(vc))
-        vals <- c(lwrtri(vals), sigma(x))
-      }
-      ret <- dplyr::tibble(
-        effect = "ran_pars",
-        group = grpnames,
-        term = c(nmvec),
-        estimate = c(vals)
-      )
+        ret <- tidy_varcov(x, rscale = rscale, conf.int = conf.int, conf.level = conf.level)
 
-      if (conf.int) {
-        ii <- intervals(x, which = "var-cov")$reStruct
-        trfun <- function(z) {
-          nm <- rownames(z)
-          nm <- sub(
-            "\\(", "_",
-            sub(
-              ")$", "",
-              sub(",", ".", nm)
-            )
-          )
-          ## ugh, swap order of vars in cor term
-          corterms <- grepl("^cor", nm)
-          re <- "_([^\\.]+)\\.(.+)$"
-          nm[corterms] <-
-            sub(re, "_\\2.\\1", nm[corterms],
-              perl = TRUE
-            )
-          return(dplyr::tibble(
-            term = nm, conf.low = z[, "lower"],
-            conf.high = z[, "upper"]
-          ))
-        }
-        ci <- dplyr::bind_rows(lapply(ii, trfun), .id = "group")
-        if (rscale != "sdcor") {
-          ## FIXME: transform/recompute from scratch
-          warning("confidence intervals for ran pars only available on sdcor scale")
-          ci$conf.low <- ci$conf.high <- NA
-        }
-        ## FIXME: also do confint on residual
-        ret <- dplyr::full_join(ret, ci, by = c("group", "term"))
-      }
-    } ## if not multi-level model
+    }
     if (attr(x$modelStruct, 'fixedSigma')) {
       mask_residual <- ret$group == "Residual"
       if (sum(mask_residual) != 1) {
@@ -311,20 +230,33 @@ augment.lme <- function(x, data = x$data, newdata, ...) {
 }
 
 
+#' @importFrom dplyr across mutate
 #' @export
-as.data.frame.ranef.lme <- function(x, row.names, optional, ...) {
+as.data.frame.ranef.lme <- function(x, row.names, optional=TRUE,
+                                    stringsAsFactors = FALSE, ...) {
+  where <- NULL
+  ## see https://github.com/r-lib/tidyselect/issues/201
+  ## this is better than utils::globalVariables() which is global ...
+
     group <- term <- level <- estimate <- NULL ## NSE arg checking
     if (!missing(row.names)) stop(sQuote("row.names"),
                                   "  argument not implemented")
-    if (!missing(optional)) stop(sQuote("optional"),
-                                 " argument not implemented")
     if (length(list(...)>0)) warning("additional arguments ignored")
-    melt <- function(x) purrr::map_dfr(as.list(x), ~tibble(level = rownames(x), estimate = .), .id = "term")
+    ## see ?as.data.frame: optional==FALSE corresponds to check.names==TRUE
+    ## "check_unique" is the tibble default, "universal" behaves like
+    ##  check.names == TRUE
+    name_repair <- if (optional) "check_unique" else "universal"
+    melt <- function(x) purrr::map_dfr(as.list(x),
+                 ~tibble(level = rownames(x), estimate = .), .id = "term")
     grps <- attr(x, "grpNames")
     if (length(grps)==1) x <- list(x)
     mm <- purrr::map(x, melt)
-    purrr::map2_dfr(.x = mm, .y = grps, ~mutate(.x, group = .y)) %>%
-        dplyr::select(group, term, level, estimate)
+    res <- purrr::map2_dfr(.x = mm, .y = grps, ~mutate(.x, group = .y)) %>%
+      dplyr::select(group, term, level, estimate)
+    if (stringsAsFactors) {
+      res <- res %>% dplyr::mutate(across(where(is.character), as.factor))
+    }
+    return(res)
 }
 
 #' @rdname nlme_tidiers
@@ -405,9 +337,9 @@ augment.gls <- function(x, data = nlme::getData(x), newdata, ...) {
 # varFunc tidiers ####
 
 #' Tidy variance structure for the \code{nlme} package.
-#' 
+#'
 #' Returns a tibble with the following columns:
-#' \itemize{
+#' \describe{
 #' \item{group}{type of varFunc, along with the right hand side of the formula
 #'   in parentheses e.g. \code{"varExp(age | Sex)"}.}
 #' \item{term}{terms included in the formula of the variance model, specifically
@@ -434,7 +366,7 @@ augment.gls <- function(x, data = nlme::getData(x), newdata, ...) {
 #'     median(as.integer(ChickWeight_arbitrary_group$Chick))
 #'   )
 #' ChickWeight_arbitrary_group$group_arb <- c("low", "high")[ChickWeight_arbitrary_group$group_arb_n]
-#' 
+#'
 #' fit_with_fixed <-
 #'   lme(
 #'     weight ~ Diet * Time,
@@ -456,7 +388,7 @@ tidy.varFunc <- function(x, ...) {
   aux <- coef(x, unconstrained = FALSE, allCoef = TRUE)
   if (length(aux) == 0) {
     warning(
-      "Variance function structure of class", class(x)[1], 
+      "Variance function structure of class", class(x)[1],
       "with no parameters, or uninitialized"
     )
     return(tibble::tibble())
@@ -495,3 +427,87 @@ tidy.varComb <- function(x, ...) {
     ))
   ret
 }
+
+tidy_varcov <- function(x, rscale = "sdcor", conf.int = TRUE, conf.level = 0.95, ...) {
+    vc <- nlme::getVarCov(x)
+    grplen <- attr(x$modelStruct$reStruct, "plen")
+    ran_prefix <- switch(rscale,
+        vcov = c("var", "cov"),
+        sdcor = c("sd", "cor")
+      )
+      ## construct appropriate sd/cor names from dimnames
+      nmvec <- outer(
+        colnames(vc), rownames(vc),
+        function(x, y) {
+          ifelse(x == y,
+            sprintf(
+              "%s_%s",
+              ran_prefix[1],
+              x
+            ),
+            sprintf(
+              "%s_%s.%s",
+              ran_prefix[2],
+              x, y
+            )
+          )
+        }
+      )
+      lwrtri <- function(x) {
+        x[lower.tri(x, diag = TRUE)]
+      }
+      nmvec <- c(
+        lwrtri(nmvec),
+        sprintf("%s_%s", ran_prefix[1], "Observation")
+      )
+      grpnames <- c(rep(names(grplen), grplen), "Residual")
+      if (rscale == "vcov") {
+        vals <- c(lwrtri(vc), sigma(x)^2)
+      } else {
+        vals <- cov2cor(vc)
+        diag(vals) <- sqrt(diag(vc))
+        vals <- c(lwrtri(vals), sigma(x))
+      }
+      ret <- dplyr::tibble(
+        effect = "ran_pars",
+        group = grpnames,
+        term = c(nmvec),
+        estimate = c(vals)
+        )
+    if (conf.int) {
+        ii <- intervals(x, level = conf.level, which = "var-cov")$reStruct
+        trfun <- function(z) {
+          nm <- rownames(z)
+          nm <- sub(
+            "\\(", "_",
+            sub(
+              ")$", "",
+              sub(",", ".", nm)
+            )
+          )
+          ## ugh, swap order of vars in cor term
+          corterms <- grepl("^cor", nm)
+          re <- "_([^\\.]+)\\.(.+)$"
+          nm[corterms] <-
+            sub(re, "_\\2.\\1", nm[corterms],
+              perl = TRUE
+            )
+          return(dplyr::tibble(
+            term = nm, conf.low = z[, "lower"],
+            conf.high = z[, "upper"]
+          ))
+        }
+        ci <- dplyr::bind_rows(lapply(ii, trfun), .id = "group")
+        if (rscale != "sdcor") {
+          ## FIXME: transform/recompute from scratch
+          warning("confidence intervals for ran pars only available on sdcor scale")
+          ci$conf.low <- ci$conf.high <- NA
+        }
+        ## FIXME: also do confint on residual
+        ## (Once FIXED, also FIX the test "tidy works on nlme/lme fits" in test
+        ## file tests/testthat/test-nlme.R, see FIXME note in the test itself).
+        ret <- dplyr::full_join(ret, ci, by = c("group", "term"))
+    } ## if conf.int  
+    return(ret)
+}
+
