@@ -95,9 +95,50 @@ if (require(glmmTMB, quietly = TRUE)
           tidy(m1, conf.int=TRUE)[,c("estimate", "conf.low","conf.high")]))
       ## this test is not reliable ... not sure what else to test?
       ## expect_equal(unname(res), c(-0.196,0.196), tolerance=1e-4)
-      expect_equal(unname(glance(m1)[,c(1,3)]),
-                   unname(tibble(100L, 98L)))
+      expect_equal(unname(glance(m1)[,c("nobs","df.residual")]),
+                   unname(tibble::tibble(100L, 98L)))
   })
 
-} ## if require(glmmTMB)
+  test_that("confint with multiple REs", {
+      if (packageVersion("glmmTMB") > "1.1.3" && requireNamespace("lme4")) {
+          dd <- expand.grid(r = 1:10, a = factor(1:2), b = factor(1:3),
+                            f = factor(1:5), g = factor(1:6))
+        dd$y <- simulate(
+            seed = 101,
+            ~ 1 + (a|f) + (b|g),
+            newdata = dd,
+            newparams = list(beta = 1,
+                             theta = rep(1,9),
+                             sigma = 1),
+            family = gaussian)[[1]]
+          res <- glmmTMB(y~ 1 + (a+0|f) + (b+0|g), data = dd)
+          td <- tidy(res, conf.int = TRUE)
+          check_tidy(
+              td, 11, 10,
+              c("effect", "component", "group", "term", "estimate", "std.error", 
+                "statistic", "p.value", "conf.low", "conf.high"))
+          } ## require lme4 (for simulate)
+  }
+  )
+  if (requireNamespace("lme4")) {
+      ## GH #136
+      data("sleepstudy", package = "lme4")
+      ## FIXME: speed up by storing this?
+      ## warning because using bogus example
+      suppressWarnings(fm3ZIP <-
+                           glmmTMB(round(Reaction) ~ Days + (1|Subject), family=poisson,
+                                   ziformula=~(1|Subject),
+                                   data = sleepstudy))
+      t1 <- tidy(fm3ZIP, conf.int = TRUE, component = "cond", effect = "ran_pars")
+      t2 <- tidy(fm3ZIP, conf.int = TRUE, effect = "ran_pars")
+      expect_identical(nrow(t1), 1L)
+      expect_identical(nrow(t2), 2L)
+  } ## if requireNamespace("lme4")
 
+
+  test_that("vv without RE in conditional model", {
+      m <- glmmTMB(mpg ~ hp, data = mtcars,
+                   dispformula = ~ 1 + (1|cyl), family = gaussian)
+      expect_equal(nrow(tidy(m)), 2L)
+   })
+} ## if require(glmmTMB)
